from _typeshed import Incomplete
from cf_extension_core.dynamo_table_creator import DynamoTableCreator as DynamoTableCreator
from cloudformation_cli_python_lib.interface import BaseResourceHandlerRequest as _BaseResourceHandlerRequest
from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource, Table
from typing import Any, Optional, Type, TYPE_CHECKING

logger: Incomplete

class ResourceBase:
    T: Incomplete
    _request: Incomplete
    _db_resource: Incomplete
    _primary_identifier: Incomplete
    _type_name: Incomplete
    def __init__(
        self,
        db_resource: DynamoDBServiceResource,
        request: _BaseResourceHandlerRequest,
        type_name: str,
        primary_identifier: Optional[str] = ...,
    ) -> None: ...

    class _ResourceData:
        _HELPER_KEY: str
        T: Incomplete
        @staticmethod
        def _model_to_string(model: T) -> str: ...
        @staticmethod
        def _model_from_string(modelstr: str, class_type: Type[T]) -> T: ...

        class DecryptionException(Exception):
            def __init__(self, *args: Any) -> None: ...

    def _current_time(self) -> str: ...
    def _get_primary_identifier(self) -> str: ...
    def _not_found_check(self) -> None: ...
    def _dynamo_db_table(self) -> Table: ...
    def _db_item_insert_without_overwrite(self, model: T) -> None: ...
    def _db_item_update_model(self, model: T) -> None: ...
    def _db_get_item(self) -> Any: ...
    def _db_item_exists(self) -> bool: ...
    def _db_item_get_model(self, model_type: Type[T]) -> T: ...
    def _db_item_list_primary_identifiers_for_cr_type(self) -> list[str]: ...
    def _db_item_delete(self, best_effort: bool = ...) -> None: ...



import typing
from _typeshed import Incomplete
from cf_extension_core.interface import (
    CustomResourceHelpers as CustomResourceHelpers,
    create_resource as create_resource,
    delete_resource as delete_resource,
    generate_dynamodb_resource as generate_dynamodb_resource,
    initialize_handler as initialize_handler,
    list_resource as list_resource,
    package_logging_config as package_logging_config,
    read_resource as read_resource,
    update_resource as update_resource,
)
from cf_extension_core.resource_create import ResourceCreate as ResourceCreate
from cf_extension_core.resource_delete import ResourceDelete as ResourceDelete
from cf_extension_core.resource_list import ResourceList as ResourceList
from cf_extension_core.resource_read import ResourceRead as ResourceRead
from cf_extension_core.resource_update import ResourceUpdate as ResourceUpdate
from cloudformation_cli_python_lib.boto3_proxy import SessionProxy as SessionProxy
from cloudformation_cli_python_lib.interface import ProgressEvent
from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource
from typing import Any, MutableMapping, TypeVar, Generic, TYPE_CHECKING

T = TypeVar("T")
K = TypeVar("K")
LOG: Incomplete

class BaseHandler(Generic[T, K]):
    _session: SessionProxy
    _request: K
    _callback_context: MutableMapping[str, Any]
    _db_resource: DynamoDBServiceResource
    _type_name: str
    _total_timeout_in_minutes: int
    def __init__(
        self,
        session: SessionProxy,
        request: K,
        callback_context: MutableMapping[str, Any],
        type_name: str,
        db_resource: DynamoDBServiceResource,
        total_timeout_in_minutes: int,
        cf_core_log_level: int = ...,
    ) -> None: ...
    @property
    def session(self) -> SessionProxy: ...
    @property
    def request(self) -> K: ...
    @property
    def callback_context(self) -> MutableMapping[str, Any]: ...
    @property
    def db_resource(self) -> DynamoDBServiceResource: ...
    @property
    def type_name(self) -> str: ...
    @property
    def total_timeout_in_minutes(self) -> int: ...
    def save_model_to_callback(self, data: T) -> None: ...
    def get_model_from_callback(self, cls: typing.Type[T] = ...) -> T: ...
    def _class_type_t(self, cls: typing.Type[T] = ...) -> T: ...
    def handler_is_timing_out(self) -> bool: ...
    def create_resource(self) -> ResourceCreate: ...
    def update_resource(self, primary_identifier: str) -> ResourceUpdate: ...
    def list_resource(self) -> ResourceList: ...
    def read_resource(self, primary_identifier: str) -> ResourceRead: ...
    def delete_resource(self, primary_identifier: str) -> ResourceDelete: ...
    def return_in_progress_event(self, message: str = ..., call_back_delay_seconds: int = ...) -> ProgressEvent: ...
    def return_success_event(self, resource_model: T, message: str = ...) -> ProgressEvent: ...
    def return_success_delete_event(self, message: str = ...) -> ProgressEvent: ...
    def validate_identifier(self, identifier: typing.Optional[str]) -> str: ...
    def _stabilize(
        self,
        function: typing.Callable[[], bool],
        sleep_seconds: int = 3,
        callback_delay: int = 1,
        callback_message: str = "",
    ) -> typing.Union[None, ProgressEvent]: ...
    def run_call_chain_with_stabilization(
        self,
        func_list: list[typing.Callable[[], bool]],
        func_retries_sleep_time: int = 3,
        callback_delay: int = 1,
        callback_message: str = "",
    ) -> typing.Union[ProgressEvent, None]: ...



import cf_extension_core.resource_create as _resource_create
import cf_extension_core.resource_delete as _resource_delete
import cf_extension_core.resource_list as _resource_list
import cf_extension_core.resource_read as _resource_read
import cf_extension_core.resource_update as _resource_update
from _typeshed import Incomplete
from cf_extension_core.constants import DynamoDBValues as DynamoDBValues
from cf_extension_core.custom_resource_helpers import CustomResourceHelpers as CustomResourceHelpers
from cf_extension_core.dynamo_table_creator import DynamoTableCreator as DynamoTableCreator
from cloudformation_cli_python_lib.boto3_proxy import SessionProxy as _SessionProxy
from cloudformation_cli_python_lib.interface import BaseResourceHandlerRequest as _BaseResourceHandlerRequest
from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource as _DynamoDBServiceResource
from typing import Any, MutableMapping, Optional, TYPE_CHECKING

LOG: Incomplete

def generate_dynamodb_resource(session_proxy: Optional[_SessionProxy]) -> _DynamoDBServiceResource: ...
def create_resource(
    request: _BaseResourceHandlerRequest, type_name: str, db_resource: _DynamoDBServiceResource
) -> _resource_create.ResourceCreate: ...
def update_resource(
    primary_identifier: str, type_name: str, request: _BaseResourceHandlerRequest, db_resource: _DynamoDBServiceResource
) -> _resource_update.ResourceUpdate: ...
def delete_resource(
    primary_identifier: str, type_name: str, request: _BaseResourceHandlerRequest, db_resource: _DynamoDBServiceResource
) -> _resource_delete.ResourceDelete: ...
def read_resource(
    primary_identifier: str, type_name: str, request: _BaseResourceHandlerRequest, db_resource: _DynamoDBServiceResource
) -> _resource_read.ResourceRead: ...
def list_resource(
    type_name: str, request: _BaseResourceHandlerRequest, db_resource: _DynamoDBServiceResource
) -> _resource_list.ResourceList: ...
def initialize_handler(callback_context: MutableMapping[str, Any], total_allowed_time_in_minutes: int) -> None: ...
def package_logging_config(logging_level: int) -> None: ...



from cf_extension_core.base_handler import BaseHandler as BaseHandler
from cf_extension_core.interface import (
    CustomResourceHelpers as CustomResourceHelpers,
    generate_dynamodb_resource as generate_dynamodb_resource,
)
